#! /usr/bin/perl

use strict;
use warnings FATAL => 'all';

use Time::Local;

my $NODE_TYPE = 0;
my $NODE_YEAR = 1;

my $LINK_NODE1 = 0;
my $LINK_NODE2 = 1;
my $LINK_TYPE = 2;

my %nodes = ();
my %links = ();
my %author_hash = ();
my %location_hash = ();
my %citation_hash = ();
while( my $line = <STDIN> ) {
	chomp( $line );
	if( $line =~ /<OBJECT ID=\"(\d+)\"\/>/ ) {
		my $node_id = $1;
		if( !defined( $nodes{$node_id} ) ) {
			$nodes{$node_id} = [];
		}
	} elsif( $line =~ /<LINK ID=\"(\d+)\" O1-ID=\"(\d+)\" O2-ID=\"(\d+)\"\/>/ ) {
		my $link_id = $1;
		my $node1_id = $2;
		my $node2_id = $3;
		if( !defined( $links{$link_id} ) ) {
			$links{$link_id} = [];
		}
		$links{$link_id}->[$LINK_NODE1] = $node1_id;
		$links{$link_id}->[$LINK_NODE2] = $node2_id;
	} elsif( $line =~ /<ATTRIBUTE NAME=\"object-type\".*/ ) {
		while( my $array_ref = read_attribute() ) {
			my $id = $array_ref->[0];
			my $value = $array_ref->[1];
			$nodes{$id}->[$NODE_TYPE] = $value;
		}
	} elsif( $line =~ /<ATTRIBUTE NAME=\"year\".*/ ) {
		while( my $array_ref = read_attribute() ) {
			my $id = $array_ref->[0];
			my $value = $array_ref->[1];
			$nodes{$id}->[$NODE_YEAR] = $value;
		}
	} elsif( $line =~ /<ATTRIBUTE NAME=\"in-year\".*/ ) {
		while( my $array_ref = read_attribute() ) {
			my $id = $array_ref->[0];
			my $value = $array_ref->[1];
			my $node1_id = $links{$id}->[$LINK_NODE1];
			my $node2_id = $links{$id}->[$LINK_NODE2];
			$nodes{$node1_id}->[$NODE_YEAR] = $value;
		}
	} elsif( $line =~ /<ATTRIBUTE NAME=\"link-type\".*/ ) {
		while( my $array_ref = read_attribute() ) {
			my $id = $array_ref->[0];
			my $value = $array_ref->[1];
			$links{$id}->[$LINK_TYPE] = $value;
			
			my $node1_id = $links{$id}->[$LINK_NODE1];
			my $node2_id = $links{$id}->[$LINK_NODE2];
			if( $value eq "author-of" ) {
				if( !defined( $author_hash{$node2_id} ) ) {
					$author_hash{$node2_id} = [];
				}
				push( @{$author_hash{$node2_id}}, $node1_id );
			} elsif( $value eq "cites" ) {
				if( !defined( $citation_hash{$node1_id} ) ) {
					$citation_hash{$node1_id} = [];
				}
				push( @{$citation_hash{$node1_id}}, $node2_id );
			} elsif( $value eq "in-proceedings" || $value eq "in-journal" || $value eq "in-collection" ) {
				if( !defined( $location_hash{$node1_id} ) ) {
					$location_hash{$node1_id} = $node2_id;
				}
			}
		}
	}
}

foreach my $paper_id (keys(%author_hash)) {
	my $year;
	if( defined( $nodes{$paper_id}->[$NODE_YEAR] ) ) {
		$year = $nodes{$paper_id}->[$NODE_YEAR];
	} elsif( defined( $location_hash{$paper_id} ) && defined( $nodes{$location_hash{$paper_id}}->[$NODE_YEAR] ) ) {
		$year = $nodes{$location_hash{$paper_id}}->[$NODE_YEAR];
	} else {
		next;
	}
	if( !defined( $citation_hash{$paper_id} ) ) {
		next;
	}
	my $author_array_ref = $author_hash{$paper_id};
	foreach my $cited_paper_id (@{$citation_hash{$paper_id}}) {
		my $cited_author_array_ref = $author_hash{$cited_paper_id};
		foreach my $author1_id (@{$author_array_ref}) {
			foreach my $author2_id (@{$cited_author_array_ref}) {
				if( $author1_id != $author2_id ) {
					print STDOUT "$year,$author1_id,$author2_id\n";
				}
			}
		}
	}
}

sub read_attribute {
	my $line = <STDIN>;
	if( $line =~ /<\/ATTRIBUTE>/ ) {
		return 0;
	} else {
		$line =~ /<ATTR-VALUE ITEM-ID=\"(\d+)\">/;
		my $id = $1;
		$line = <STDIN>;
		$line =~ /<COL-VALUE>(.*)<\/COL-VALUE>/;
		my $value = $1;
		$line = <STDIN>;
		return [$id,$value];
	}
}